@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -69,12 +69,12 @@ import com.openexchange.server.ServiceException;
 
 /**
  * {@link MALPollDBUtility} - DB utilities for MAL poll bundle.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class MALPollDBUtility {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(MALPollDBUtility.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MALPollDBUtility.class));
 
     private static final int CHUNK_SIZE = 100;
 
@@ -87,7 +87,7 @@ public final class MALPollDBUtility {
 
     /**
      * Inserts the mail IDs associated with specified hash.
-     * 
+     *
      * @param hash The hash
      * @param mailIds The new mail IDs
      * @param cid The context ID
@@ -96,22 +96,11 @@ public final class MALPollDBUtility {
      */
     public static void insertMailIDs(final UUID hash, final Set<String> mailIds, final int cid) throws PushException {
         final DatabaseService databaseService = getDBService();
-        final Connection writableConnection;
-        try {
-            writableConnection = databaseService.getWritable(cid);
-            writableConnection.setAutoCommit(false); // BEGIN
-        } catch (final DBPoolingException e) {
-            throw new PushException(e);
-        } catch (final SQLException e) {
-            throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
-        }
+        final Connection writableConnection = getReadWriteConnection(cid, databaseService, false);
         try {
             final StringBuilder sb = new StringBuilder(CHUNK_SIZE * 16).append(SQL_INSERT_PREFIX);
             insert0(cid, hash, mailIds, CHUNK_SIZE, sb, writableConnection);
             writableConnection.commit(); // COMMIT
-        } catch (final SQLException e) {
-            rollback(writableConnection);
-            throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
         } catch (final Exception e) {
             rollback(writableConnection);
             throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
@@ -149,7 +138,7 @@ public final class MALPollDBUtility {
                     sb.append(values);
                     final String delim = ", ";
                     for (int i = j + 1; i < limit; i++) {
-                        sb.append(delim).append(values); // , VALUES (?, ?)
+                        sb.append(delim).append(values); // , VALUES (?, UNHEX(REPLACE(?,'-','')), ?)
                     }
                     stmt = writableConnection.prepareStatement(sb.toString());
                 }
@@ -201,7 +190,7 @@ public final class MALPollDBUtility {
 
     /**
      * Replaces the mail IDs associated with specified hash.
-     * 
+     *
      * @param hash The hash
      * @param newIds The new mail IDs
      * @param delIds The deleted mail IDs
@@ -210,24 +199,13 @@ public final class MALPollDBUtility {
      */
     public static void replaceMailIDs(final UUID hash, final Set<String> newIds, final Set<String> delIds, final int cid) throws PushException {
         final DatabaseService databaseService = getDBService();
-        final Connection writableConnection;
-        try {
-            writableConnection = databaseService.getWritable(cid);
-            writableConnection.setAutoCommit(false); // BEGIN
-        } catch (final DBPoolingException e) {
-            throw new PushException(e);
-        } catch (final SQLException e) {
-            throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
-        }
+        final Connection writableConnection = getReadWriteConnection(cid, databaseService, false);
         try {
             deletet0(cid, hash, delIds, CHUNK_SIZE, writableConnection);
             final StringBuilder sb = new StringBuilder(CHUNK_SIZE * 16).append(SQL_INSERT_PREFIX);
             insert0(cid, hash, newIds, CHUNK_SIZE, sb, writableConnection);
 
             writableConnection.commit(); // COMMIT
-        } catch (final SQLException e) {
-            rollback(writableConnection);
-            throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
         } catch (final Exception e) {
             rollback(writableConnection);
             throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
@@ -239,7 +217,7 @@ public final class MALPollDBUtility {
 
     /**
      * Drops the mail IDs associated with specified hash and the hash itself, too.
-     * 
+     *
      * @param cid The context ID
      * @param user The user ID
      * @throws PushException If a database resource could not be acquired
@@ -258,7 +236,7 @@ public final class MALPollDBUtility {
             deleteEntries(cid, con, uuids);
             deleteUserData(cid, con, user);
             con.commit(); // COMMIT
-        } catch (final SQLException e) {
+        } catch (final Exception e) {
             rollback(con);
             throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
         } finally {
@@ -267,8 +245,7 @@ public final class MALPollDBUtility {
         }
     }
 
-    private static List<UUID> getUserUUIDs(int cid, Connection con, int user) throws SQLException {
-        final List<UUID> uuids = new ArrayList<UUID>();
+    private static List<UUID> getUserUUIDs(final int cid, final Connection con, final int user) throws SQLException {
         PreparedStatement stmt = null;
         ResultSet rs = null;
         try {
@@ -277,17 +254,18 @@ public final class MALPollDBUtility {
             stmt.setInt(1, cid);
             stmt.setInt(2, user);
             rs = stmt.executeQuery();
+            final List<UUID> uuids = new ArrayList<UUID>();
             while (rs.next()) {
                 uuids.add(toUUID(rs.getBytes(1)));
             }
+            return uuids;
         } finally {
             MALPollDBUtility.closeSQLStuff(rs);
             MALPollDBUtility.closeSQLStuff(stmt);
         }
-        return uuids;
     }
 
-    private static void deleteEntries(int cid, Connection con, List<UUID> uuids) throws SQLException {
+    private static void deleteEntries(final int cid, final Connection con, final List<UUID> uuids) throws SQLException {
         if (uuids.isEmpty()) {
             return;
         }
@@ -305,7 +283,41 @@ public final class MALPollDBUtility {
         }
     }
 
-    private static void deleteUserData(int cid, Connection con, int user) throws SQLException {
+    /**
+     * Drops the user data.
+     * <p>
+     * Prefer {@link #dropMailIDs(int, int)} instead.
+     *
+     * @param cid The context ID
+     * @param user The user ID
+     * @throws PushException If a database resource could not be acquired
+     */
+    public static void deleteUserData(final int cid, final int user) throws PushException {
+        final DatabaseService databaseService = getDBService();
+        final Connection con;
+        try {
+            con = databaseService.getForUpdateTask(cid);
+        } catch (final DBPoolingException e) {
+            throw new PushException(e);
+        }
+        boolean rollback = true;
+        try {
+            con.setAutoCommit(false);
+            deleteUserData(cid, con, user);
+            con.commit(); // COMMIT
+            rollback = false;
+        } catch (final Exception e) {
+            throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
+        } finally {
+            if (rollback) {
+                rollback(con);
+            }
+            autocommit(con);
+            databaseService.backForUpdateTask(cid, con);
+        }
+    }
+
+    private static void deleteUserData(final int cid, final Connection con, final int user) throws SQLException {
         PreparedStatement stmt = null;
         try {
             /*
@@ -322,7 +334,7 @@ public final class MALPollDBUtility {
 
     /**
      * Gets all mail IDs associated with specified hash.
-     * 
+     *
      * @param hash The hash
      * @param cid The context ID
      * @return The mail IDs associated with specified hash
@@ -330,12 +342,7 @@ public final class MALPollDBUtility {
      */
     public static Set<String> getMailIDs(final UUID hash, final int cid) throws PushException {
         final DatabaseService databaseService = getDBService();
-        final Connection readableConnection;
-        try {
-            readableConnection = databaseService.getReadOnly(cid);
-        } catch (final DBPoolingException e) {
-            throw new PushException(e);
-        }
+        final Connection readableConnection = getReadOnlyConnection(cid, databaseService);
         try {
             PreparedStatement stmt = null;
             ResultSet rs = null;
@@ -363,7 +370,7 @@ public final class MALPollDBUtility {
 
     /**
      * Gets the hash for specified keys.
-     * 
+     *
      * @param cid The context ID
      * @param user The user ID
      * @param accountId The account ID
@@ -373,12 +380,7 @@ public final class MALPollDBUtility {
      */
     public static UUID getHash(final int cid, final int user, final int accountId, final String fullname) throws PushException {
         final DatabaseService databaseService = getDBService();
-        final Connection readableConnection;
-        try {
-            readableConnection = databaseService.getReadOnly(cid);
-        } catch (final DBPoolingException e) {
-            throw new PushException(e);
-        }
+        final Connection readableConnection = getReadOnlyConnection(cid, databaseService);
         try {
             PreparedStatement stmt = null;
             ResultSet rs = null;
@@ -409,7 +411,7 @@ public final class MALPollDBUtility {
 
     /**
      * Generates and inserts a new hash for specified keys.
-     * 
+     *
      * @param cid The context ID
      * @param user The user ID
      * @param accountId The account ID
@@ -419,12 +421,7 @@ public final class MALPollDBUtility {
      */
     public static UUID insertHash(final int cid, final int user, final int accountId, final String fullname) throws PushException {
         final DatabaseService databaseService = getDBService();
-        final Connection writableConnection;
-        try {
-            writableConnection = databaseService.getWritable(cid);
-        } catch (final DBPoolingException e) {
-            throw new PushException(e);
-        }
+        final Connection writableConnection = getReadWriteConnection(cid, databaseService);
         try {
             PreparedStatement stmt = null;
             try {
@@ -449,27 +446,11 @@ public final class MALPollDBUtility {
         }
     }
 
-    /**
-     * Gets the {@link DatabaseService database service} from service registry.
-     * 
-     * @return The database service
-     * @throws PushException If database service is not available
-     */
-    private static DatabaseService getDBService() throws PushException {
-        final DatabaseService databaseService;
-        try {
-            databaseService = MALPollServiceRegistry.getServiceRegistry().getService(DatabaseService.class, true);
-        } catch (final ServiceException e) {
-            throw new PushException(e);
-        }
-        return databaseService;
-    }
-
     private static final int UUID_BYTE_LENGTH = 16;
 
     /**
      * Generates a new {@link UUID} instance from specified byte array.
-     * 
+     *
      * @param bytes The byte array
      * @return A new {@link UUID} instance
      * @throws IllegalArgumentException If passed byte array is <code>null</code> or its length is not 16
@@ -492,9 +473,52 @@ public final class MALPollDBUtility {
         return new UUID(msb, lsb);
     }
 
+    /**
+     * Gets the {@link DatabaseService database service} from service registry.
+     *
+     * @return The database service
+     * @throws PushException If database service is not available
+     */
+    private static DatabaseService getDBService() throws PushException {
+        final DatabaseService databaseService;
+        try {
+            databaseService = MALPollServiceRegistry.getServiceRegistry().getService(DatabaseService.class, true);
+        } catch (final ServiceException e) {
+            throw new PushException(e);
+        }
+        return databaseService;
+    }
+
+    private static Connection getReadWriteConnection(final int cid, final DatabaseService databaseService) throws PushException {
+        return getReadWriteConnection(cid, databaseService, true);
+    }
+
+    private static Connection getReadWriteConnection(final int cid, final DatabaseService databaseService, final boolean autoCommit) throws PushException {
+        try {
+            if (autoCommit) {
+                return databaseService.getWritable(cid);
+            }
+            final Connection writableConnection = databaseService.getWritable(cid);
+            writableConnection.setAutoCommit(false); // BEGIN
+            return writableConnection;
+        } catch (final DBPoolingException e) {
+            throw new PushException(e);
+        } catch (final SQLException e) {
+            throw PushExceptionCodes.UNEXPECTED_ERROR.create(e, e.getMessage());
+        }
+    }
+
+    private static Connection getReadOnlyConnection(final int cid, final DatabaseService databaseService) throws PushException {
+        try {
+            return databaseService.getReadOnly(cid);
+        } catch (final DBPoolingException e) {
+            throw new PushException(e);
+        }
+    }
+
     /**
      * Rolls-back specified connection.
-     * 
+     *
      * @param con The connection to roll back.
      */
     private static void rollback(final Connection con) {
@@ -510,7 +534,7 @@ public final class MALPollDBUtility {
 
     /**
      * Convenience method to set the auto-commit of a connection to <code>true</code>.
-     * 
+     *
      * @param con The connection that should go into auto-commit mode.
      */
     private static void autocommit(final Connection con) {
@@ -526,7 +550,7 @@ public final class MALPollDBUtility {
 
     /**
      * Closes the {@link ResultSet}.
-     * 
+     *
      * @param result <code>null</code> or a {@link ResultSet} to close.
      */
     private static void closeSQLStuff(final ResultSet result) {
@@ -541,7 +565,7 @@ public final class MALPollDBUtility {
 
     /**
      * Closes the {@link Statement}.
-     * 
+     *
      * @param stmt <code>null</code> or a {@link Statement} to close.
      */
     private static void closeSQLStuff(final Statement stmt) {
