@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,6 +54,9 @@ import java.util.HashSet;
 import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.atomic.AtomicBoolean;
+import com.openexchange.context.ContextService;
+import com.openexchange.groupware.contexts.Context;
+import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.mail.MailException;
 import com.openexchange.mail.MailField;
 import com.openexchange.mail.MailSortField;
@@ -73,12 +76,12 @@ import com.openexchange.timer.TimerService;
 
 /**
  * {@link MALPollPushListener} - The MAL poll {@link PushListener}.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class MALPollPushListener implements PushListener {
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(MALPollPushListener.class);
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MALPollPushListener.class));
 
     private static final boolean DEBUG_ENABLED = LOG.isDebugEnabled();
 
@@ -95,7 +98,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Gets the account ID constant.
-     * 
+     *
      * @return The account ID constant
      */
     public static int getAccountId() {
@@ -104,7 +107,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Sets static folder fullname.
-     * 
+     *
      * @param folder The folder fullname
      */
     public static void setFolder(final String folder) {
@@ -113,7 +116,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Gets static folder fullname.
-     * 
+     *
      * @return The folder fullname
      */
     public static String getFolder() {
@@ -122,7 +125,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Sets static period milliseconds.
-     * 
+     *
      * @param periodMillis The period milliseconds
      */
     public static void setPeriodMillis(final long periodMillis) {
@@ -131,7 +134,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Initializes a new {@link MALPollPushListener}.
-     * 
+     *
      * @param session The needed session to obtain and connect mail access instance
      * @param startTimerTask <code>true</code> to start a timer task for this listener
      * @return A new {@link MALPollPushListener}.
@@ -160,7 +163,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Initializes a new {@link MALPollPushListener}.
-     * 
+     *
      * @param session The needed session to obtain and connect mail access instance
      * @param ignoreOnGlobal <code>true</code> to ignore during global run
      */
@@ -169,8 +172,8 @@ public final class MALPollPushListener implements PushListener {
         running = new AtomicBoolean();
         this.session = session;
         this.ignoreOnGlobal = ignoreOnGlobal;
-        this.userId = session.getUserId();
-        this.contextId = session.getContextId();
+        userId = session.getUserId();
+        contextId = session.getContextId();
     }
 
     @Override
@@ -183,7 +186,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Opens this listener (if {@link #isIgnoreOnGlobal()} returns <code>false</code>).
-     * 
+     *
      * @throws PushException If listener cannot be opened
      */
     public void open() throws PushException {
@@ -224,7 +227,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Checks whether to ignore this listener on global run.
-     * 
+     *
      * @return <code>true</code> to ignore this listener on global run; otherwise <code>false</code>
      */
     public boolean isIgnoreOnGlobal() {
@@ -233,7 +236,7 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Check for new mails
-     * 
+     *
      * @throws PushException If check for new mails fails
      */
     public void checkNewMail() throws PushException {
@@ -247,6 +250,17 @@ public final class MALPollPushListener implements PushListener {
             }
             return;
         }
+        try {
+            final ContextService contextService = MALPollServiceRegistry.getServiceRegistry().getService(ContextService.class, true);
+            final Context context = contextService.getContext(contextId);
+            if (context.isReadOnly()) {
+                return;
+            }
+        } catch (final ServiceException e) {
+            throw new PushException(e);
+        } catch (final ContextException e) {
+            throw new PushException(e);
+        }
         try {
             final MailService mailService = MALPollServiceRegistry.getServiceRegistry().getService(MailService.class, true);
             if (started) {
@@ -371,9 +385,9 @@ public final class MALPollPushListener implements PushListener {
         mailAccess.connect();
         try {
             final String fullname = folder;
-            final Set<String> uidSet = new HashSet<String>(mailAccess.getFolderStorage().getFolder(fullname).getMessageCount());
             final MailMessage[] messages =
                 mailAccess.getMessageStorage().searchMessages(fullname, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, null, FIELDS);
+            final Set<String> uidSet = new HashSet<String>(messages.length);
             for (final MailMessage mailMessage : messages) {
                 uidSet.add(mailMessage.getMailId());
             }
@@ -389,11 +403,20 @@ public final class MALPollPushListener implements PushListener {
 
     /**
      * Checks if this listener has been started.
-     * 
+     *
      * @return <code>true</code> if this listener has been started; otherwise <code>false</code>
      */
     public boolean isStarted() {
         return started;
     }
 
+    /**
+     * Gets the session.
+     *
+     * @return The session
+     */
+    public Session getSession() {
+        return session;
+    }
+
 }
