@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,7 +57,7 @@ import com.openexchange.push.PushException;
 public final class MALPollPushListenerRunnable implements Runnable {
 
     private static final org.apache.commons.logging.Log LOG =
-        org.apache.commons.logging.LogFactory.getLog(MALPollPushListenerRunnable.class);
+        com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(MALPollPushListenerRunnable.class));
 
     private final MALPollPushListener listener;
 
